import putki.discover as discover


def test_root_default():
    assert discover.root() == ''


def test_root_cwd_is_root():
    assert discover.root('.') == ''


def test_root_within_not_existing():
    assert discover.root('non-existing-path') == ''


def test_root_within_is_without():
    assert discover.root('/') == ''



import pathlib
from typing import no_type_check

import yaml
from git.exc import GitCommandError, InvalidGitRepositoryError, NoSuchPathError
from git.repo import Repo
from putki import ENCODING


@no_type_check
def root(within: str | pathlib.Path = '.') -> str:
    """Detect the root folder for tasks."""
    try:
        repo = Repo(within, search_parent_directories=True)
        try:
            repo_root = repo.git.rev_parse(show_toplevel=True)
            start_here = pathlib.Path(repo_root)
            for path in start_here.rglob('*'):
                if path.name == 'tasks' and path.is_dir():
                    return str(path)
            return ''
        except GitCommandError:
            return ''
    except (InvalidGitRepositoryError, NoSuchPathError):
        return ''


@no_type_check
def tasks(below: str | pathlib.Path = '.') -> dict[str, list[dict[str, str]]]:
    """Collect the tasks below by mapping the paths to lists of key value string maps."""
    jobs: dict[str, list[dict[str, str]]] = {}
    start_here = pathlib.Path(below)
    for path in start_here.rglob('**/*'):
        if path.is_file() and path.stat().st_size and path.suffix.lower() in ('.yaml', '.yml'):
            with open(path, 'rt', encoding=ENCODING) as handle:
                data = yaml.safe_load(handle)
            if data and data.get('tasks', None):
                jobs[str(path.parent)] = data['tasks']
    return jobs



"""Command line interface for pipeline (Finnish: putki) - discovering and executing a specific task description."""
import logging
import pathlib
import sys

import typer

import putki.api as api
from putki import APP_NAME, DEFAULT_STRUCTURE_NAME, QUIET, __version__ as APP_VERSION, log

app = typer.Typer(
    add_completion=False,
    context_settings={'help_option_names': ['-h', '--help']},
    no_args_is_help=True,
)

DocumentRoot = typer.Option(
    '',
    '-d',
    '--document-root',
    help='Root of the document tree to visit. Optional\n(default: positional tree root value)',
)
StructureName = typer.Option(
    DEFAULT_STRUCTURE_NAME,
    '-s',
    '--structure',
    help='structure mapping file (default: {DEFAULT_STRUCTURE_NAME})',
)
TargetName = typer.Option(
    '',
    '-t',
    '--target',
    help='target document key',
)
FacetName = typer.Option(
    '',
    '-f',
    '--facet',
    help='facet key of target document',
)
Verbosity = typer.Option(
    False,
    '-v',
    '--verbose',
    help='Verbose output (default is False)',
)
Strictness = typer.Option(
    False,
    '-s',
    '--strict',
    help='Ouput noisy warnings on console (default is False)',
)
OutputPath = typer.Option(
    '',
    '-o',
    '--output-path',
    help='Path to output unambiguous content to - like when ejecting a template',
)


@app.callback(invoke_without_command=True)
def callback(
    version: bool = typer.Option(
        False,
        '-V',
        '--version',
        help='Display the application version and exit',
        is_eager=True,
    )
) -> None:
    """
    Pipeline (Finnish: putki) - discovering and executing a specific task description.
    """
    if version:
        typer.echo(f'{APP_NAME} version {APP_VERSION}')
        raise typer.Exit()


def _verify_call_vector(
    doc_root: str, doc_root_pos: str, verbose: bool, strict: bool
) -> tuple[int, str, str, dict[str, bool]]:
    """DRY"""
    doc = doc_root.strip()
    if not doc and doc_root_pos:
        doc = doc_root_pos
    if not doc:
        print('Document tree root required', file=sys.stderr)
        return 2, 'Document tree root required', '', {}

    doc_root_path = pathlib.Path(doc)
    if doc_root_path.exists():
        if not doc_root_path.is_dir():
            print(f'requested tree root at ({doc}) is not a folder', file=sys.stderr)
            return 2, f'requested tree root at ({doc}) is not a folder', '', {}
    else:
        print(f'requested tree root at ({doc}) does not exist', file=sys.stderr)
        return 2, f'requested tree root at ({doc}) does not exist', '', {}

    options = {
        'quiet': QUIET and not verbose and not strict,
        'strict': strict,
        'verbose': verbose,
    }
    if verbose:
        logging.getLogger().setLevel(logging.DEBUG)
    return 0, '', doc, options


@app.command('verify')
def verify(  # noqa
    doc_root_pos: str = typer.Argument(''),
    doc_root: str = DocumentRoot,
    structure: str = StructureName,
    target: str = TargetName,
    facet: str = FacetName,
    verbose: bool = Verbosity,
    strict: bool = Strictness,
) -> int:
    """
    Verify the structure definition against the file system.
    """
    code, message, doc, options = _verify_call_vector(doc_root, doc_root_pos, verbose, strict)
    if code:
        log.error(message)
        return code

    return sys.exit(
        api.verify(doc_root=doc, structure_name=structure, target_key=target, facet_key=facet, options=options)
    )


@app.command('version')
def app_version() -> None:
    """
    Display the application version and exit.
    """
    callback(True)
